import matplotlib
import matplotlib.pyplot as plt
import numpy as np
from matplotlib import colors
from matplotlib.ticker import PercentFormatter
 
plt.rcParams.update({'font.size': 15}) 
matplotlib.rcParams['pdf.fonttype'] = 42
matplotlib.rcParams['ps.fonttype'] = 42

def plot_cdf(data1, data2, data3, data4, x_label, legend, savefig, bins):
    fig, ax = plt.subplots()
    count_1, bins_count_1 = np.histogram(data1, bins=bins)
    pdf_1 = count_1 / sum(count_1)
    cdf_1 = np.cumsum(pdf_1)
    
    count_2, bins_count_2 = np.histogram(data2, bins=bins)
    pdf_2 = count_2 / sum(count_2)
    cdf_2 = np.cumsum(pdf_2)

    count_3, bins_count_3 = np.histogram(data3, bins=bins)
    pdf_3 = count_3 / sum(count_3)
    cdf_3 = np.cumsum(pdf_3)

    count_4, bins_count_4 = np.histogram(data4, bins=bins)
    pdf_4 = count_4 / sum(count_4)
    cdf_4 = np.cumsum(pdf_4)
    
    ax.plot(bins_count_1[1:], cdf_1, color='r',)
    ax.plot(bins_count_2[1:], cdf_2, color='r', linestyle="--")
    ax.plot(bins_count_3[1:], cdf_3, color='b')
    ax.plot(bins_count_4[1:], cdf_4, color='b', linestyle="--")

    plt.xlabel(x_label)
    plt.ylabel("CDF")
    ax.legend(legend, frameon=False)
    plt.tight_layout()
    plt.show()
    #plt.savefig(savefig, format='pdf')

#
X = np.arange(0, 13, 1)
g_opt = [4,4,2,2,3,2,3,3,4,4,4,4,3,3,4,4,3,3,4,5,3,3,3,3,3]
g_fea = [12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12]
z_opt = [4,4,2,2,3,2,3,3,4,4,4,4,3,3,4,4,3,3,4,5,3,3,3,3,3]
z_fea = [4,4,2,2,3,2,3,3,12,4,12,11,11,11,11,12,12,12,12,11,12,12,12,12,12]
assert len(g_opt) == len(g_fea)
assert len(z_opt) == len(z_fea)
legend = ['Gurobi Optimal', 'Gurobi Feasible', 'Z3 Optimal', 'Z3 Feasible']
plot_cdf(g_opt, g_fea, z_opt, z_fea, "Num of stages", legend, "./Gurobi_Z3_res.pdf", X)


#
X = np.arange(0, 3, 0.1)
g_time_opt = [0.133,0.095,0.102,0.102,0.127,0.131,0.151,0.117,0.169,0.165,0.19,0.178,0.18,0.202,0.241,0.198,0.159,0.156,0.169,0.181,0.145,0.162,0.215,0.174]
z_time_opt = [0.251,0.218,0.201,0.210,0.232,0.234,0.241,0.223,0.81,0.781,1.137,0.967,0.88,0.889,1.033,0.963,0.845,0.922,0.946,1.086,0.733,1.578,2.544,1.53]
assert len(g_time_opt) == len(z_time_opt)
g_time_fea = [0.137,0.117,0.105,0.103,0.124,0.135,0.146,0.119,0.161,0.163,0.180,0.168,0.173,0.182,0.220,0.183,0.142,0.146,0.154,0.170,0.148,0.168,0.201,0.149]
z_time_fea = [0.273,0.244,0.225,0.226,0.254,0.253,0.272,0.241,1.805,0.501,0.639,0.502,0.506,0.523,0.585,0.553,0.459,0.447,0.531,0.600,0.462,1.578,0.798,0.595]
assert len(g_time_fea) == len(z_time_fea)
legend = ['Gurobi Optimal', 'Gurobi Feasible', 'Z3 Optimal', 'Z3 Feasible']
plot_cdf(g_time_opt, g_time_fea, z_time_opt, z_time_fea, "Running time" ,legend, "./Gurobi_Z3_time.pdf", X)

'''
#
X = np.arange(0, 13, 1)
g_opt = [4,4,2,2,3,2,3,3,4,4,4,4,3,3,4,4,3,3,4,5,3,3,3,3,3]
g_fea = [12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12]
assert len(g_opt) == len(g_fea)
legend = ['Gurobi Optimal solution', 'Gurobi Feasible solution']
plot_cdf(g_opt, g_fea, "Num of stages", legend, "./Gurobi_res.pdf", X)

#
X = np.arange(0, 13, 1)
z_opt = [4,4,2,2,3,2,3,3,4,4,4,4,3,3,4,4,3,3,4,5,3,3,3,3,3]
z_fea = [4,4,2,2,3,2,3,3,12,4,12,11,11,11,11,12,12,12,12,11,12,12,12,12,12]
assert len(z_opt) == len(z_fea)
legend = ['Z3 Optimal solution', 'Z3 Feasible solution']
plot_cdf(z_opt, z_fea, "Num of stages", legend, "./Z3_res.pdf", X)

#
X = np.arange(0, 3, 0.1)
g_time_opt = [0.133,0.095,0.102,0.102,0.127,0.131,0.151,0.117,0.169,0.165,0.19,0.178,0.18,0.202,0.241,0.198,0.159,0.156,0.169,0.181,0.145,0.162,0.215,0.174]
z_time_opt = [0.251,0.218,0.201,0.210,0.232,0.234,0.241,0.223,0.81,0.781,1.137,0.967,0.88,0.889,1.033,0.963,0.845,0.922,0.946,1.086,0.733,1.578,2.544,1.53]
assert len(g_time_opt) == len(z_time_opt)
legend = ['Gurobi Optimal mode', 'Z3 Optimal mode']
plot_cdf(g_time_opt, z_time_opt, legend, "./g_z_time_opt.pdf", X)

X = np.arange(0, 3, 0.1)
g_time_fea = [0.137,0.117,0.105,0.103,0.124,0.135,0.146,0.119,0.161,0.163,0.180,0.168,0.173,0.182,0.220,0.183,0.142,0.146,0.154,0.170,0.148,0.168,0.201,0.149]
z_time_fea = [0.273,0.244,0.225,0.226,0.254,0.253,0.272,0.241,1.805,0.501,0.639,0.502,0.506,0.523,0.585,0.553,0.459,0.447,0.531,0.600,0.462,1.578,0.798,0.595]
assert len(g_time_fea) == len(z_time_fea)
legend = ['Gurobi Feasible mode', 'Z3 Feasible mode']
plot_cdf(g_time_fea, z_time_fea, legend, "./g_z_time_fea.pdf", X)
'''
